from typing import List

from entities.raw_data_models import CompanyEmissions
from climate_risk.errors import ServiceError
from storage.emissions_cafeprod import EmissionsCAFEPROD


class Coverage():
    """
    the purpose of this class is to do basic coverage checks for particular fields on a particular dataset
    example: emissions scope 1 coverage for equity portfolio
    returns coverage figures by count and by weight
    """
    def __init__(self) -> None:
        # TODO: fix this
        self.nav = None
        # self.nav = FundValuation().get_value()
        self.emissions = None

    def _get_data(self):
        self.emissions = EmissionsCAFEPROD().get()
        return self.emissions

    def calc_cov(self, emissions: List[CompanyEmissions], check_field: str):
        try:
            nans = {
                i.organisation_code: {"emission": getattr(i, check_field), "nav_nok": i.nav_nok}
                for i in emissions
                if getattr(i, check_field) is None
            }
        except AttributeError:
            raise ServiceError(msg=f"Coverage calculation failed. This field doesn't exist in the passed object: {check_field}")
        # calculate value portion not covered by emissions data
        nans_total_valye = sum([d["nav_nok"] for d in nans.values()])
        return {
            "count": {
                "description": "count of missing data",
                "value": len(nans),
                "percentage": round(len(nans)/len(emissions) * 100, 2),
                "full_load": nans
            },
            "invested_value": {
                "description": f"total value not covered by {check_field}",
                "value": nans_total_valye,
                "percentage": round(nans_total_valye / self.nav["eq_nav_nok"]["NOK"] * 100, 2)
                }
        }

    def coverage_check(self, fields: List[str] = ["emission_scope_1", "emission_scope_2"]):
        # TODO: plan to exapand to fi
        emissions = self._get_data()
        report_fields_cov = {}
        for field in fields:
            report_fields_cov.update({field: self.calc_cov(emissions, field)})
        return report_fields_cov
